@@ -88,6 +88,8 @@ void ghb_scale_configure(signal_user_data_t *ud, char *name, double val,
 void ghb_update_summary_info(signal_user_data_t *ud);
 void ghb_set_title_settings(signal_user_data_t *ud, GhbValue *settings);
 void ghb_browse_uri(signal_user_data_t *ud, const gchar *uri);
+void ghb_break_pts_duration(gint64 ptsDuration,
+                            gint *hh, gint *mm, gdouble *ss);
 void ghb_break_duration(gint64 duration, gint *hh, gint *mm, gint *ss);
 
 #endif // _CALLBACKS_H_
