@@ -832,17 +832,25 @@ static int hb_detelecine_init( hb_filter_object_t * filter,
     pv->input      = *init;
     pv->pullup_ctx = ctx = pullup_alloc_context();
 
-    ctx->junk_left = ctx->junk_right  = 1;
-    ctx->junk_top  = ctx->junk_bottom = 4;
+    // "Skip" array [top, bottom, left, right]
+    int top, bottom, left, right;
+
+    left = right = ctx->junk_left = ctx->junk_right  = 1;
+    top = bottom = ctx->junk_top  = ctx->junk_bottom = 4;
     ctx->strict_breaks = -1;
     ctx->metric_plane  = 0;
     ctx->parity = -1;
 
-    // "Skip" array [top, bottom, left, right]
-    hb_dict_extract_int(&ctx->junk_top, filter->settings, "skip-top");
-    hb_dict_extract_int(&ctx->junk_bottom, filter->settings, "skip-bottom");
-    hb_dict_extract_int(&ctx->junk_left, filter->settings, "skip-left");
-    hb_dict_extract_int(&ctx->junk_right, filter->settings, "skip-right");
+    hb_dict_extract_int(&top,    filter->settings, "skip-top");
+    hb_dict_extract_int(&bottom, filter->settings, "skip-bottom");
+    hb_dict_extract_int(&left,   filter->settings, "skip-left");
+    hb_dict_extract_int(&right,  filter->settings, "skip-right");
+    // Enforce safety zones
+    ctx->junk_top    = top    > ctx->junk_top    ? top    : ctx->junk_top;
+    ctx->junk_bottom = bottom > ctx->junk_bottom ? bottom : ctx->junk_bottom;
+    ctx->junk_left   = left   > ctx->junk_left   ? left   : ctx->junk_left;
+    ctx->junk_right  = right  > ctx->junk_right  ? right  : ctx->junk_right;
+
     hb_dict_extract_int(&ctx->strict_breaks, filter->settings, "strict-breaks");
     hb_dict_extract_int(&ctx->metric_plane, filter->settings, "plane");
     hb_dict_extract_int(&ctx->parity, filter->settings, "parity");
