@@ -100,6 +100,8 @@ static hb_triplet_t hb_qsv_h265_levels[] =
     { NULL,                            },
 };
 
+#define MFX_IMPL_VIA_MASK(impl) (0x0f00 & (impl))
+
 // check available Intel Media SDK version against a minimum
 #define HB_CHECK_MFX_VERSION(MFX_VERSION, MAJOR, MINOR) \
     (MFX_VERSION.Major == MAJOR  && MFX_VERSION.Minor >= MINOR)
@@ -928,7 +930,7 @@ hb_list_t* hb_qsv_load_plugins(hb_qsv_info_t *info, mfxSession session, mfxVersi
 
     if (HB_CHECK_MFX_VERSION(version, 1, 8))
     {
-        if (info->codec_id == MFX_CODEC_HEVC)
+        if (info->codec_id == MFX_CODEC_HEVC && !(qsv_hardware_generation(hb_get_cpu_platform()) < QSV_G5))
         {
             if (HB_CHECK_MFX_VERSION(version, 1, 15) &&
                 qsv_implementation_is_hardware(info->implementation))
@@ -994,7 +996,9 @@ const char* hb_qsv_decode_get_codec_name(enum AVCodecID codec_id)
 int hb_qsv_decode_is_enabled(hb_job_t *job)
 {
     return ((job != NULL && job->qsv.decode) &&
-            (job->title->video_decode_support & HB_DECODE_SUPPORT_QSV));
+            (job->title->video_decode_support & HB_DECODE_SUPPORT_QSV)) &&
+            (job->vcodec == HB_VCODEC_QSV_H265 ? (qsv_hardware_generation(hb_get_cpu_platform()) >= QSV_G5) : 1) &&
+            (job->vcodec == HB_VCODEC_QSV_H265_10BIT ? (qsv_hardware_generation(hb_get_cpu_platform()) >= QSV_G6) : 1);
 }
 
 static int hb_dxva2_device_check();
@@ -2297,6 +2301,7 @@ extern EncQSVFramesContext hb_enc_qsv_frames_ctx;
 AVBufferRef *hb_hw_device_ctx = NULL;
 char *qsv_device = NULL;
 mfxHDL device_manager_handle = NULL;
+mfxIMPL device_impl;
 
 #if defined(_WIN32) || defined(__MINGW32__)
 // Direct X
@@ -2318,13 +2323,13 @@ static int hb_dxva2_device_create9(HMODULE d3dlib, UINT adapter, IDirect3D9 **d3
 {
     pDirect3DCreate9 *createD3D = (pDirect3DCreate9 *)hb_dlsym(d3dlib, "Direct3DCreate9");
     if (!createD3D) {
-        hb_error("Failed to locate Direct3DCreate9");
+        hb_error("hb_dxva2_device_create9: failed to locate Direct3DCreate9");
         return -1;
     }
 
     IDirect3D9 *d3d9 = createD3D(D3D_SDK_VERSION);
     if (!d3d9) {
-        hb_error("Failed to create IDirect3D object");
+        hb_error("hb_dxva2_device_create9: createD3D failed");
         return -1;
     }
     *d3d9_out = d3d9;
@@ -2338,14 +2343,14 @@ static int hb_dxva2_device_create9ex(HMODULE d3dlib, UINT adapter, IDirect3D9 **
     pDirect3DCreate9Ex *createD3DEx = (pDirect3DCreate9Ex *)hb_dlsym(d3dlib, "Direct3DCreate9Ex");
     if (!createD3DEx)
     {
-        hb_error("Failed to locate Direct3DCreate9Ex");
+        hb_error("hb_dxva2_device_create9ex: failed to locate Direct3DCreate9Ex");
         return -1;
     }
 
     hr = createD3DEx(D3D_SDK_VERSION, &d3d9ex);
     if (FAILED(hr))
     {
-        hb_error("Failed to create IDirect3DEx object");
+        hb_error("hb_dxva2_device_create9ex: createD3DEx failed %d", hr);
         return -1;
     }
     *d3d9_out = (IDirect3D9 *)d3d9ex;
@@ -2360,7 +2365,7 @@ static int hb_d3d11va_device_check()
     dxgilib = hb_dlopen("dxgi.dll");
     if (!d3dlib || !dxgilib)
     {
-        hb_error("Failed to load d3d11.dll and dxgi.dll");
+        hb_error("hb_d3d11va_device_check: failed to load d3d11.dll and dxgi.dll");
         return -1;
     }
 
@@ -2370,7 +2375,7 @@ static int hb_d3d11va_device_check()
     mCreateDXGIFactory = (HB_PFN_CREATE_DXGI_FACTORY)hb_dlsym(dxgilib, "CreateDXGIFactory1");
 
     if (!mD3D11CreateDevice || !mCreateDXGIFactory) {
-        hb_error("Failed to load D3D11 library functions");
+        hb_error("hb_d3d11va_device_check: failed to locate D3D11CreateDevice and CreateDXGIFactory1 functions");
         return -1;
     }
 
@@ -2386,13 +2391,13 @@ static int hb_d3d11va_device_check()
 
         hr = mD3D11CreateDevice(pAdapter, D3D_DRIVER_TYPE_UNKNOWN, NULL, D3D11_CREATE_DEVICE_VIDEO_SUPPORT, NULL, 0, D3D11_SDK_VERSION, &g_pd3dDevice, NULL, NULL);
         if (FAILED(hr)) {
-            hb_error("D3D11CreateDevice returned error");
+            hb_error("hb_d3d11va_device_check: D3D11CreateDevice returned %d", hr);
             continue;
         }
 
         hr = IDXGIAdapter2_GetDesc(pAdapter, &adapterDesc);
         if (FAILED(hr)) {
-            hb_error("IDXGIAdapter2_GetDesc returned error");
+            hb_error("hb_d3d11va_device_check: IDXGIAdapter2_GetDesc returned %d", hr);
             continue;
         }
 
@@ -2401,7 +2406,7 @@ static int hb_d3d11va_device_check()
 
         if (adapterDesc.VendorId == 0x8086) {
             IDXGIFactory2_Release(pDXGIFactory);
-            hb_log("D3D11: QSV adapter with id %d has been found", adapter_id - 1);
+            hb_log("hb_d3d11va_device_check: QSV adapter with id %d has been found", adapter_id - 1);
             return adapter_id - 1;
         }
     }
@@ -2411,36 +2416,36 @@ static int hb_d3d11va_device_check()
 
 static int hb_dxva2_device_check()
 {
+    HRESULT hr;
     HMODULE d3dlib = NULL;
     IDirect3D9 *d3d9 = NULL;
     D3DADAPTER_IDENTIFIER9 identifier;
     D3DADAPTER_IDENTIFIER9 *d3dai = &identifier;
     UINT adapter = D3DADAPTER_DEFAULT;
-    int err = 0;
 
     d3dlib = hb_dlopen("d3d9.dll");
     if (!d3dlib)
     {
-        hb_error("Failed to load D3D9 library");
+        hb_error("hb_dxva2_device_check: failed to load d3d9 library");
         return -1;
     }
 
     if (hb_dxva2_device_create9ex(d3dlib, adapter, &d3d9) < 0)
     {
         // Retry with "classic" d3d9
-        err = hb_dxva2_device_create9(d3dlib, adapter, &d3d9);
-        if (err < 0)
+        hr = hb_dxva2_device_create9(d3dlib, adapter, &d3d9);
+        if (hr < 0)
         {
-            err = -1;
+            hr = -1;
             goto clean_up;
         }
     }
 
-    UINT adapter_count = IDirect3D9_GetAdapterCount(d3d9);
-    if (FAILED(IDirect3D9_GetAdapterIdentifier(d3d9, D3DADAPTER_DEFAULT, 0, d3dai)))
+    hr = IDirect3D9_GetAdapterIdentifier(d3d9, D3DADAPTER_DEFAULT, 0, d3dai);
+    if (FAILED(hr))
     {
-        hb_error("Failed to get Direct3D adapter identifier");
-        err = -1;
+        hb_error("hb_dxva2_device_check: IDirect3D9_GetAdapterIdentifier failed");
+        hr = -1;
         goto clean_up;
     }
 
@@ -2449,12 +2454,12 @@ static int hb_dxva2_device_check()
     {
         if(d3dai->VendorId != intel_id)
         {
-            hb_error("D3D9: adapter that was found does not support QSV. It is required for zero-copy QSV path");
-            err = -1;
+            hb_error("hb_dxva2_device_check: adapter that was found does not support QSV. It is required for zero-copy QSV path");
+            hr = -1;
             goto clean_up;
         }
     }
-    err = 0;
+    hr = 0;
 
 clean_up:
     if (d3d9)
@@ -2463,7 +2468,7 @@ clean_up:
     if (d3dlib)
         hb_dlclose(d3dlib);
 
-    return err;
+    return hr;
 }
 
 static HRESULT lock_device(
@@ -2544,16 +2549,8 @@ int hb_qsv_replace_surface_mid(const QSVMid *mid, mfxFrameSurface1 *surface)
     int ret = hb_qsv_find_surface_idx(hb_enc_qsv_frames_ctx.mids, hb_enc_qsv_frames_ctx.nb_mids, mid);
     if (ret < 0)
     {
-        ret = hb_qsv_find_surface_idx(hb_enc_qsv_frames_ctx.mids2, hb_enc_qsv_frames_ctx.nb_mids, mid);
-        if (ret < 0)
-        {
-            hb_error("encqsv: Surface with MemId=%p has not been found in the pool\n", mid);
-            return -1;
-        }
-        else
-        {
-            surface->Data.MemId = &hb_enc_qsv_frames_ctx.mids2[ret];
-        }
+        hb_error("encqsv: Surface with MemId=%p has not been found in the pool\n", mid);
+        return -1;
     }
     else
     {
@@ -2567,16 +2564,8 @@ int hb_qsv_release_surface_from_pool(const QSVMid *mid)
     int ret = hb_qsv_find_surface_idx(hb_enc_qsv_frames_ctx.mids, hb_enc_qsv_frames_ctx.nb_mids, mid);
     if (ret < 0)
     {
-        ret = hb_qsv_find_surface_idx(hb_enc_qsv_frames_ctx.mids2, hb_enc_qsv_frames_ctx.nb_mids, mid);
-        if (ret < 0)
-        {
-            hb_error("encqsv: Surface with MemId=%p has not been found in the pool\n", mid);
-            return -1;
-        }
-        else
-        {
-            ff_qsv_atomic_dec(&hb_enc_qsv_frames_ctx.pool2[ret]);
-        }
+        hb_error("encqsv: Surface with MemId=%p has not been found in the pool\n", mid);
+        return -1;
     }
     else
     {
@@ -2593,9 +2582,6 @@ void hb_qsv_get_free_surface_from_pool(const int start_index, const int end_inde
     AVHWFramesContext *frames_ctx = (AVHWFramesContext*)hb_enc_qsv_frames_ctx.hw_frames_ctx->data;
     AVQSVFramesContext *frames_hwctx = frames_ctx->hwctx;
 
-    AVHWFramesContext *frames_ctx2 = (AVHWFramesContext*)hb_enc_qsv_frames_ctx.hw_frames_ctx2->data;
-    AVQSVFramesContext *frames_hwctx2 = frames_ctx2->hwctx;
-
     // find the first available surface in the pool
     int count = 0;
     while(1)
@@ -2622,22 +2608,6 @@ void hb_qsv_get_free_surface_from_pool(const int start_index, const int end_inde
             }
         }
 
-        for(int i = start_index; i < end_index; i++)
-        {
-            if(hb_enc_qsv_frames_ctx.pool2[i] == 0)
-            {
-                mid = &hb_enc_qsv_frames_ctx.mids2[i];
-                output_surface = &frames_hwctx2->surfaces[i];
-                if(output_surface->Data.Locked == 0)
-                {
-                    *out_mid = mid;
-                    *out_surface = output_surface;
-                    ff_qsv_atomic_inc(&hb_enc_qsv_frames_ctx.pool2[i]);
-                    return;
-                }
-            }
-        }
-
         count++;
     }
 }
@@ -2661,48 +2631,47 @@ hb_buffer_t* hb_qsv_copy_frame(AVFrame *frame, hb_qsv_context *qsv_ctx)
 
     QSVMid *mid = NULL;
     mfxFrameSurface1* output_surface = NULL;
-    AVHWFramesContext *frames_ctx = (AVHWFramesContext*)hb_enc_qsv_frames_ctx.hw_frames_ctx->data;
-    AVQSVFramesContext *frames_hwctx = frames_ctx->hwctx;
 
     hb_qsv_get_free_surface_from_pool(0, HB_POOL_SURFACE_SIZE - HB_POOL_ENCODER_SIZE, &mid, &output_surface);
 
-    // Get D3DDeviceManger handle from Media SDK
-    mfxHandleType handle_type;
-
-    static const mfxHandleType handle_types[] = {
-        MFX_HANDLE_VA_DISPLAY,
-        MFX_HANDLE_D3D11_DEVICE,
-        MFX_HANDLE_D3D9_DEVICE_MANAGER
-    };
-
-    int i;
-
     AVHWDeviceContext    *device_ctx = (AVHWDeviceContext*)hb_hw_device_ctx->data;
     AVQSVDeviceContext *device_hwctx = device_ctx->hwctx;
     mfxSession        parent_session = device_hwctx->session;
 
     if (device_manager_handle == NULL)
     {
-        for (i = 0; i < 3; i++)
+        mfxHandleType handle_type;
+        int err = MFXQueryIMPL(parent_session, &device_impl);
+        if (err != MFX_ERR_NONE)
         {
-            int err = MFXVideoCORE_GetHandle(parent_session, handle_types[i], &device_manager_handle);
-            if (err == MFX_ERR_NONE)
-            {
-                handle_type = handle_types[i];
-                break;
-            }
-            device_manager_handle = NULL;
+            hb_error("hb_qsv_copy_frame: no impl could be retrieved");
+            return out;
         }
 
-        if (!device_manager_handle)
+        if (MFX_IMPL_VIA_D3D11 == MFX_IMPL_VIA_MASK(device_impl))
         {
-            hb_error("No supported hw handle could be retrieved "
+            handle_type = MFX_HANDLE_D3D11_DEVICE;
+        }
+        else if (MFX_IMPL_VIA_D3D9 == MFX_IMPL_VIA_MASK(device_impl))
+        {
+            handle_type = MFX_HANDLE_D3D9_DEVICE_MANAGER;
+        }
+        else
+        {
+            hb_error("hb_qsv_copy_frame: unsupported impl");
+            return out;
+        }
+
+        err = MFXVideoCORE_GetHandle(parent_session, handle_type, &device_manager_handle);
+        if (err != MFX_ERR_NONE)
+        {
+            hb_error("hb_qsv_copy_frame: no supported hw handle could be retrieved "
                 "from the session\n");
             return out;
         }
     }
 
-    if (handle_type == MFX_HANDLE_D3D9_DEVICE_MANAGER)
+    if (MFX_IMPL_VIA_D3D9 == MFX_IMPL_VIA_MASK(device_impl))
     {
         IDirect3DDevice9 *pDevice = NULL;
         HANDLE handle;
@@ -2710,7 +2679,7 @@ hb_buffer_t* hb_qsv_copy_frame(AVFrame *frame, hb_qsv_context *qsv_ctx)
         HRESULT result = lock_device((IDirect3DDeviceManager9 *)device_manager_handle, 0, &pDevice, &handle);
         if (FAILED(result))
         {
-            hb_error("copy_frame qsv: LockDevice failded=%d", result);
+            hb_error("hb_qsv_copy_frame: lock_device failded %d", result);
             return out;
         }
 
@@ -2724,17 +2693,17 @@ hb_buffer_t* hb_qsv_copy_frame(AVFrame *frame, hb_qsv_context *qsv_ctx)
         result = IDirect3DDevice9_StretchRect(pDevice, input_surface->Data.MemId, 0, mid->handle, 0, D3DTEXF_LINEAR);
         if (FAILED(result))
         {
-            hb_error("copy_frame qsv: IDirect3DDevice9_StretchRect failded=%d", result);
+            hb_error("hb_qsv_copy_frame: IDirect3DDevice9_StretchRect failded %d", result);
             return out;
         }
         result = unlock_device((IDirect3DDeviceManager9 *)device_manager_handle, handle);
         if (FAILED(result))
         {
-            hb_error("copy_frame qsv: UnlockDevice failded=%d", result);
+            hb_error("hb_qsv_copy_frame: unlock_device failded %d", result);
             return out;
         }
-    }
-    else
+    } 
+    else if (MFX_IMPL_VIA_D3D11 == MFX_IMPL_VIA_MASK(device_impl))
     {
         ID3D11DeviceContext *device_context = NULL;
         ID3D11Device *device = (ID3D11Device *)device_manager_handle;
@@ -2749,10 +2718,15 @@ hb_buffer_t* hb_qsv_copy_frame(AVFrame *frame, hb_qsv_context *qsv_ctx)
         // replace the mem id to mem id from the pool
         output_surface->Data.MemId = mid;
         // copy input sufrace to sufrace from the pool
-        ID3D11DeviceContext_CopySubresourceRegion(device_context, mid->texture, mid->handle, 0, 0, 0, hb_enc_qsv_frames_ctx.input_texture, input_surface->Data.MemId, NULL);
+        ID3D11DeviceContext_CopySubresourceRegion(device_context, mid->texture, (uint64_t)mid->handle, 0, 0, 0, hb_enc_qsv_frames_ctx.input_texture, (uint64_t)input_surface->Data.MemId, NULL);
+    }
+    else 
+    {
+        hb_error("hb_qsv_copy_frame: incorrect mfx impl");
+        return out;
     }
 
-    out->qsv_details.frame->data[3] = output_surface;
+    out->qsv_details.frame->data[3] = (uint8_t*)output_surface;
     out->qsv_details.qsv_atom = 0;
     out->qsv_details.ctx      = qsv_ctx;
     return out;
@@ -2796,10 +2770,13 @@ static int qsv_device_init(AVCodecContext *s)
             return err;
     }
 
+    err = av_dict_set(&dict, "child_device_type", "d3d11va", 0);
+    err = av_dict_set(&dict, "vendor", "0x8086", 0);
+
     err = av_hwdevice_ctx_create(&hb_hw_device_ctx, AV_HWDEVICE_TYPE_QSV,
                                  0, dict, 0);
     if (err < 0) {
-        av_log(NULL, AV_LOG_ERROR, "Error creating a QSV device\n");
+        hb_error("qsv_device_init: error creating a QSV device %d", err);
         goto err_out;
     }
 
@@ -2840,7 +2817,7 @@ static int qsv_init(AVCodecContext *s)
 
     ret = av_hwframe_ctx_init(s->hw_frames_ctx);
     if (ret < 0) {
-        av_log(NULL, AV_LOG_ERROR, "Error initializing a QSV frame pool\n");
+        hb_error("qsv_init: av_hwframe_ctx_init failed %d", ret);
         return ret;
     }
 
@@ -2864,46 +2841,17 @@ static int qsv_init(AVCodecContext *s)
 
     ret = av_hwframe_ctx_init(enc_hw_frames_ctx);
     if (ret < 0) {
-        av_log(NULL, AV_LOG_ERROR, "Error initializing a QSV frame pool\n");
+        hb_error("qsv_init: av_hwframe_ctx_init failed %d", ret);
         return ret;
     }
-
-    enc_hw_frames_ctx = av_hwframe_ctx_alloc(hb_hw_device_ctx);
-    if (!enc_hw_frames_ctx)
-        return AVERROR(ENOMEM);
-
-    hb_enc_qsv_frames_ctx.hw_frames_ctx2 = enc_hw_frames_ctx;
-    frames_ctx   = (AVHWFramesContext*)enc_hw_frames_ctx->data;
-    frames_hwctx = frames_ctx->hwctx;
-
-    frames_ctx->width             = FFALIGN(s->coded_width,  32);
-    frames_ctx->height            = FFALIGN(s->coded_height, 32);
-    frames_ctx->format            = AV_PIX_FMT_QSV;
-    frames_ctx->sw_format         = s->sw_pix_fmt;
-    frames_ctx->initial_pool_size = HB_POOL_SURFACE_SIZE;
-    frames_hwctx->frame_type      = MFX_MEMTYPE_VIDEO_MEMORY_DECODER_TARGET;
-
-    ret = av_hwframe_ctx_init(enc_hw_frames_ctx);
-    if (ret < 0) {
-        av_log(NULL, AV_LOG_ERROR, "Error initializing a QSV frame pool\n");
-        return ret;
-    }
-
     /* allocate the memory ids for the external frames */
     av_buffer_unref(&hb_enc_qsv_frames_ctx.mids_buf);
     hb_enc_qsv_frames_ctx.mids_buf = hb_qsv_create_mids(hb_enc_qsv_frames_ctx.hw_frames_ctx);
     if (!hb_enc_qsv_frames_ctx.mids_buf)
         return AVERROR(ENOMEM);
-    av_buffer_unref(&hb_enc_qsv_frames_ctx.mids_buf2);
-    hb_enc_qsv_frames_ctx.mids_buf2 = hb_qsv_create_mids(hb_enc_qsv_frames_ctx.hw_frames_ctx2);
-    if (!hb_enc_qsv_frames_ctx.mids_buf2)
-        return AVERROR(ENOMEM);
-
     hb_enc_qsv_frames_ctx.mids    = (QSVMid*)hb_enc_qsv_frames_ctx.mids_buf->data;
-    hb_enc_qsv_frames_ctx.mids2   = (QSVMid*)hb_enc_qsv_frames_ctx.mids_buf2->data;
     hb_enc_qsv_frames_ctx.nb_mids = frames_hwctx->nb_surfaces;
     memset(hb_enc_qsv_frames_ctx.pool, 0, hb_enc_qsv_frames_ctx.nb_mids * sizeof(hb_enc_qsv_frames_ctx.pool[0]));
-    memset(hb_enc_qsv_frames_ctx.pool2, 0, hb_enc_qsv_frames_ctx.nb_mids * sizeof(hb_enc_qsv_frames_ctx.pool2[0]));
     return 0;
 }
 
@@ -2917,20 +2865,11 @@ int hb_qsv_get_buffer(AVCodecContext *s, AVFrame *frame, int flags)
 
 enum AVPixelFormat hb_qsv_get_format(AVCodecContext *s, const enum AVPixelFormat *pix_fmts)
 {
-    const enum AVPixelFormat *p;
-    int ret;
-
-    for (p = pix_fmts; *p != AV_PIX_FMT_NONE; p++) {
-        const AVPixFmtDescriptor *desc = av_pix_fmt_desc_get(*p);
-
-        if (!(desc->flags & AV_PIX_FMT_FLAG_HWACCEL))
-            break;
-
-        if(*p == AV_PIX_FMT_QSV)
-        {
-            ret = qsv_init(s);
+    while (*pix_fmts != AV_PIX_FMT_NONE) {
+        if (*pix_fmts == AV_PIX_FMT_QSV) {
+            int ret = qsv_init(s);
             if (ret < 0) {
-                av_log(NULL, AV_LOG_FATAL, "QSV hwaccel requested for input stream but cannot be initialized.\n");
+                hb_error("hb_qsv_get_format: QSV hwaccel initialization failed");
                 return AV_PIX_FMT_NONE;
             }
 
@@ -2939,15 +2878,14 @@ enum AVPixelFormat hb_qsv_get_format(AVCodecContext *s, const enum AVPixelFormat
                 if (!s->hw_frames_ctx)
                     return AV_PIX_FMT_NONE;
             }
-            break;
-        }
-        else
-        {
-            hb_error("get_format: *p != AV_PIX_FMT_QSV");
+            return AV_PIX_FMT_QSV;
         }
+
+        pix_fmts++;
     }
 
-    return *p;
+    hb_error("hb_qsv_get_format: the QSV pixel format not offered in get_format()");
+    return AV_PIX_FMT_NONE;
 }
 
 int hb_qsv_preset_is_zero_copy_enabled(const hb_dict_t *job_dict)
