@@ -1545,15 +1545,16 @@ static void do_job(hb_job_t *job)
         // decavcodec.c.  This may be different than title->pix_fmt
         // since we will likely only support planar YUV color formats.
         init.pix_fmt = AV_PIX_FMT_YUV420P;
+        init.color_range = AVCOL_RANGE_MPEG;
+
         init.color_prim = title->color_prim;
         init.color_transfer = title->color_transfer;
         init.color_matrix = title->color_matrix;
-        init.color_range = title->color_range;
         init.geometry.width = title->geometry.width;
         init.geometry.height = title->geometry.height;
 
         init.geometry.par = job->par;
-        memcpy(init.crop, title->crop, sizeof(int[4]));
+        memset(init.crop, 0, sizeof(int[4]));
         init.vrate = job->vrate;
         init.cfr = 0;
         init.grayscale = 0;
